'''
.. module:: updaters
   :platform: Linux
   :synopsis: Wrapper package module for a collection of tools required for doing intelligent and dynamic DSGE model instance updating
              at runtime. The version in this module is for the one-off updater behaviour.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


''' 
