'''
.. module:: solvers
   :platform: Linux
   :synopsis: Wrapper package for all kinds of modules necessary for finding solutions to DSGE model instances. Contains modules
              required for finding steady state solutions as well as such which are needed for solving the models dynamically.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


'''