"""Provides classes BasicAlgebra, Verbatim, and CollectingField.
"""
__docformat__ = 'restructuredtext'

from ..core import init_module
from .algebra import Algebra, SymbolicEquality
from .verbatim import Verbatim
#from .pairs import CollectingField
#from ..ring import CommutativeRing as CollectingField

    
