'''
.. module:: dsge
   :platform: Linux
   :synopsis: The dsge module is just a package which contains all of the modules required to do DSGE modelling, that is solving
              them and then doing further work with the obtained solutions, such as simulating and generating IRFs.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


'''